@@ -439,7 +439,7 @@ class MPILauncher(LocalProcessLauncher):
         help="The command line argument to the program."
     )
     n = Integer(1)
-    
+
     def __init__(self, *args, **kwargs):
         # deprecation for old MPIExec names:
         config = kwargs.get('config', {})
@@ -507,7 +507,7 @@ class DeprecatedMPILauncher(object):
         oldname = self.__class__.__name__
         newname = oldname.replace('MPIExec', 'MPI')
         self.log.warn("WARNING: %s name is deprecated, use %s", oldname, newname)
-    
+
 class MPIExecLauncher(MPILauncher, DeprecatedMPILauncher):
     """Deprecated, use MPILauncher"""
     def __init__(self, *args, **kwargs):
@@ -576,7 +576,7 @@ class SSHLauncher(LocalProcessLauncher):
     def find_args(self):
         return self.ssh_cmd + self.ssh_args + [self.location] + \
                list(map(pipes.quote, self.program + self.program_args))
-    
+
     def _send_file(self, local, remote):
         """send a single file"""
         full_remote = "%s:%s" % (self.location, remote)
@@ -593,7 +593,7 @@ class SSHLauncher(LocalProcessLauncher):
         )
         self.log.info("sending %s to %s", local, full_remote)
         check_output(self.scp_cmd + self.scp_args + [local, full_remote])
-    
+
     def send_files(self):
         """send our files (called before start)"""
         if not self.to_send:
@@ -617,7 +617,7 @@ class SSHLauncher(LocalProcessLauncher):
         local_dir = os.path.dirname(local)
         ensure_dir_exists(local_dir, 775)
         check_output(self.scp_cmd + [full_remote, local])
-    
+
     def fetch_files(self):
         """fetch remote files (called after start)"""
         if not self.to_fetch:
@@ -635,7 +635,7 @@ class SSHLauncher(LocalProcessLauncher):
             self.ssh_args.append(port)
             self.scp_args.append('-P')
             self.scp_args.append(port)
-        
+
         self.send_files()
         super(SSHLauncher, self).start()
         self.fetch_files()
@@ -647,26 +647,26 @@ class SSHLauncher(LocalProcessLauncher):
             self.process.stdin.flush()
 
 class SSHClusterLauncher(SSHLauncher, ClusterAppMixin):
-    
+
     remote_profile_dir = Unicode('', config=True,
-        help="""The remote profile_dir to use.  
-        
+        help="""The remote profile_dir to use.
+
         If not specified, use calling profile, stripping out possible leading homedir.
         """)
-    
+
     def _profile_dir_changed(self, name, old, new):
         if not self.remote_profile_dir:
             # trigger remote_profile_dir_default logic again,
             # in case it was already triggered before profile_dir was set
             self.remote_profile_dir = self._strip_home(new)
-    
+
     @staticmethod
     def _strip_home(path):
         """turns /home/you/.ipython/profile_foo into .ipython/profile_foo"""
         home = get_home_dir()
         if not home.endswith('/'):
             home = home+'/'
-        
+
         if path.startswith(home):
             return path[len(home):]
         else:
@@ -674,11 +674,11 @@ class SSHClusterLauncher(SSHLauncher, ClusterAppMixin):
 
     def _remote_profile_dir_default(self):
         return self._strip_home(self.profile_dir)
-    
+
     def _cluster_id_changed(self, name, old, new):
         if new:
             raise ValueError("cluster id not supported by SSH launchers")
-    
+
     @property
     def cluster_args(self):
         return ['--profile-dir', self.remote_profile_dir]
@@ -691,11 +691,11 @@ class SSHControllerLauncher(SSHClusterLauncher, ControllerMixin):
 
     def _controller_cmd_default(self):
         return ['ipcontroller']
-    
+
     @property
     def program(self):
         return self.controller_cmd
-    
+
     @property
     def program_args(self):
         return self.cluster_args + self.controller_args
@@ -712,10 +712,10 @@ class SSHEngineLauncher(SSHClusterLauncher, EngineMixin):
     # alias back to *non-configurable* program[_args] for use in find_args()
     # this way all Controller/EngineSetLaunchers have the same form, rather
     # than *some* having `program_args` and others `controller_args`
-    
+
     def _engine_cmd_default(self):
         return ['ipengine']
-    
+
     @property
     def program(self):
         return self.engine_cmd
@@ -723,10 +723,10 @@ class SSHEngineLauncher(SSHClusterLauncher, EngineMixin):
     @property
     def program_args(self):
         return self.cluster_args + self.engine_args
-    
+
     def _to_send_default(self):
         return [
-            (os.path.join(self.profile_dir, 'security', cf), 
+            (os.path.join(self.profile_dir, 'security', cf),
              os.path.join(self.remote_profile_dir, 'security', cf))
             for cf in ('ipcontroller-client.json', 'ipcontroller-engine.json')
         ]
@@ -737,10 +737,10 @@ class SSHEngineSetLauncher(LocalEngineSetLauncher):
     engines = Dict(config=True,
         help="""dict of engines to launch.  This is a dict by hostname of ints,
         corresponding to the number of engines to start on that host.""")
-    
+
     def _engine_cmd_default(self):
         return ['ipengine']
-    
+
     @property
     def engine_count(self):
         """determine engine count from `engines` dict"""
@@ -752,7 +752,7 @@ class SSHEngineSetLauncher(LocalEngineSetLauncher):
                 n = n['n']
             count += n
         return count
-    
+
     def start(self, n):
         """Start engines by profile or profile_dir.
         `n` is ignored, and the `engines` config property is used instead.
@@ -812,28 +812,28 @@ class SSHEngineSetLauncher(LocalEngineSetLauncher):
 class SSHProxyEngineSetLauncher(SSHClusterLauncher):
     """Launcher for calling
     `ipcluster engines` on a remote machine.
-    
+
     Requires that remote profile is already configured.
     """
-    
+
     n = Integer()
     ipcluster_cmd = List(['ipcluster'], config=True)
-    
+
     @property
     def program(self):
         return self.ipcluster_cmd + ['engines']
-    
+
     @property
     def program_args(self):
         return ['-n', str(self.n), '--profile-dir', self.remote_profile_dir]
-    
+
     def _to_send_default(self):
         return [
-            (os.path.join(self.profile_dir, 'security', cf), 
+            (os.path.join(self.profile_dir, 'security', cf),
              os.path.join(self.remote_profile_dir, 'security', cf))
             for cf in ('ipcontroller-client.json', 'ipcontroller-engine.json')
     ]
-    
+
     def start(self, n):
         self.n = n
         super(SSHProxyEngineSetLauncher, self).start()
@@ -1052,7 +1052,7 @@ class BatchSystemLauncher(BaseLauncher):
     batch_file_name = Unicode(u'batch_script', config=True,
         help="The filename of the instantiated batch script.")
     queue = Unicode(u'', config=True,
-        help="The PBS Queue.")
+        help="The batch queue.")
 
     def _queue_changed(self, name, old, new):
         self.context[name] = new
@@ -1061,10 +1061,10 @@ class BatchSystemLauncher(BaseLauncher):
     _n_changed = _queue_changed
 
     # not configurable, override in subclasses
-    # PBS Job Array regex
+    # Job Array regex
     job_array_regexp = CRegExp('')
     job_array_template = Unicode('')
-    # PBS Queue regex
+    # Queue regex
     queue_regexp = CRegExp('')
     queue_template = Unicode('')
     # The default batch template, override in subclasses
@@ -1081,7 +1081,7 @@ class BatchSystemLauncher(BaseLauncher):
         are set to something other than the default value.
         """
         return dict(n=1, queue=u'', profile_dir=u'', cluster_id=u'')
-    
+
     # the Formatter instance for rendering the templates:
     formatter = Instance(EvalFormatter, (), {})
 
@@ -1118,7 +1118,7 @@ class BatchSystemLauncher(BaseLauncher):
             self.batch_template = self.default_template
             # add jobarray or queue lines to user-specified template
             # note that this is *only* when user did not specify a template.
-            self._insert_queue_in_script()
+            self._insert_options_in_script()
             self._insert_job_array_in_script()
         script_as_string = self.formatter.format(self.batch_template, **self.context)
         self.log.debug('Writing batch script: %s', self.batch_file)
@@ -1126,11 +1126,11 @@ class BatchSystemLauncher(BaseLauncher):
             f.write(script_as_string)
         os.chmod(self.batch_file, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
 
-    def _insert_queue_in_script(self):
+    def _insert_options_in_script(self):
         """Inserts a queue if required into the batch script.
         """
         if self.queue and not self.queue_regexp.search(self.batch_template):
-            self.log.debug("adding PBS queue settings to batch script")
+            self.log.debug("adding queue settings to batch script")
             firstline, rest = self.batch_template.split('\n',1)
             self.batch_template = u'\n'.join([firstline, self.queue_template, rest])
 
@@ -1176,7 +1176,7 @@ class PBSLauncher(BatchSystemLauncher):
     submit_command = List(['qsub'], config=True,
         help="The PBS submit command ['qsub']")
     delete_command = List(['qdel'], config=True,
-        help="The PBS delete command ['qsub']")
+        help="The PBS delete command ['qdel']")
     job_id_regexp = CRegExp(r'\d+', config=True,
         help="Regular expresion for identifying the job ID [r'\d+']")
 
@@ -1213,6 +1213,109 @@ class PBSEngineSetLauncher(PBSLauncher, BatchClusterAppMixin):
 %s --profile-dir="{profile_dir}" --cluster-id="{cluster_id}"
 """%(' '.join(map(pipes.quote,ipengine_cmd_argv))))
 
+#Slurm is very similar to PBS
+
+class SlurmLauncher(BatchSystemLauncher):
+    """A BatchSystemLauncher subclass for slurm."""
+
+    submit_command = List(['sbatch'], config=True,
+        help="The slurm submit command ['sbatch']")
+    delete_command = List(['scancel'], config=True,
+        help="The slurm delete command ['scancel']")
+    job_id_regexp = CRegExp(r'\d+', config=True,
+        help="Regular expresion for identifying the job ID [r'\d+']")
+
+    account = Unicode(u"", config=True,
+        help="Slurm account to be used")
+
+    qos = Unicode(u"", config=True,
+        help="Slurm QoS to be used")
+
+    #Note: from the man page:
+    #'Acceptable time formats include "minutes", "minutes:seconds",
+    # "hours:minutes:seconds", "days-hours", "days-hours:minutes"
+    # and "days-hours:minutes:seconds".
+    timelimit = Any(u"", config=True,
+        help="Slurm timelimit to be used")
+
+    options = Unicode(u"", config=True,
+        help="Extra Slurm options")
+
+    def _account_changed(self, name, old, new):
+        self.context[name] = new
+
+    def _qos_changed(self, name, old, new):
+        self.context[name] = new
+
+    def _timelimit_changed(self, name, old, new):
+        self.context[name] = new
+
+    def _options_changed(self, name, old, new):
+        self.context[name] = new
+
+    batch_file = Unicode(u'')
+
+    job_array_regexp = CRegExp('#SBATCH\W+(?:--ntasks|-n)[\w\d\-\$]+')
+    job_array_template = Unicode('''#SBATCH --ntasks={n}''')
+
+    queue_regexp = CRegExp('#SBATCH\W+(?:--partition|-p)\W+\$?\w+')
+    queue_template = Unicode('#SBATCH --partition={queue}')
+
+    account_regexp = CRegExp('#SBATCH\W+(?:--account|-A)\W+\$?\w+')
+    account_template = Unicode('#SBATCH --account={account}')
+
+    qos_regexp = CRegExp('#SBATCH\W+--qos\W+\$?\w+')
+    qos_template = Unicode('#SBATCH --qos={qos}')
+
+    timelimit_regexp = CRegExp('#SBATCH\W+(?:--time|-t)\W+\$?\w+')
+    timelimit_template = Unicode('#SBATCH --time={timelimit}')
+
+    def _insert_options_in_script(self):
+        """Insert 'partition' (slurm name for queue), 'account', 'time' and other options if necessary"""
+        if self.queue and not self.queue_regexp.search(self.batch_template):
+            self.log.debug("adding slurm queue settings to batch script")
+            firstline, rest = self.batch_template.split('\n',1)
+            self.batch_template = u'\n'.join([firstline, self.queue_template, rest])
+
+        if self.account and not self.account_regexp.search(self.batch_template):
+            self.log.debug("adding slurm account settings to batch script")
+            firstline, rest = self.batch_template.split('\n',1)
+            self.batch_template = u'\n'.join([firstline, self.account_template, rest])
+
+        if self.qos and not self.qos_regexp.search(self.batch_template):
+            self.log.debug("adding Slurm qos settings to batch script")
+            firstline, rest = self.batch_template.split('\n', 1)
+            self.batch_template = u'\n'.join([firstline, self.qos_template, rest])
+
+        if self.timelimit and not self.timelimit_regexp.search(self.batch_template):
+            self.log.debug("adding slurm time limit settings to batch script")
+            firstline, rest = self.batch_template.split('\n',1)
+            self.batch_template = u'\n'.join([firstline, self.timelimit_template, rest])
+
+
+class SlurmControllerLauncher(SlurmLauncher, BatchClusterAppMixin):
+    """Launch a controller using Slurm."""
+    batch_file_name = Unicode(u'slurm_controller.sbatch', config=True,
+        help="batch file name for the controller job.")
+    default_template= Unicode("""#!/bin/sh
+#SBATCH --job-name=ipy-controller-{cluster_id}
+#SBATCH --ntasks=1
+%s --log-to-file --profile-dir="{profile_dir}" --cluster-id="{cluster_id}"
+"""%(' '.join(map(pipes.quote, ipcontroller_cmd_argv))))
+
+    def start(self):
+        """Start the controller by profile or profile_dir."""
+        return super(SlurmControllerLauncher, self).start(1)
+
+
+class SlurmEngineSetLauncher(SlurmLauncher, BatchClusterAppMixin):
+    """Launch Engines using Slurm"""
+    batch_file_name = Unicode(u'slurm_engine.sbatch', config=True,
+        help="batch file name for the engine(s) job.")
+    default_template= Unicode(u"""#!/bin/sh
+#SBATCH --job-name=ipy-engine-{cluster_id}
+srun %s --profile-dir="{profile_dir}" --cluster-id="{cluster_id}"
+""" % (' '.join(map(pipes.quote, ipengine_cmd_argv))))
 
 #SGE is very similar to PBS
 
@@ -1321,15 +1424,15 @@ class HTCondorLauncher(BatchSystemLauncher):
     """A BatchSystemLauncher subclass for HTCondor.
 
     HTCondor requires that we launch the ipengine/ipcontroller scripts rather
-    that the python instance but otherwise is very similar to PBS.  This is because 
-    HTCondor destroys sys.executable when launching remote processes - a launched 
+    that the python instance but otherwise is very similar to PBS.  This is because
+    HTCondor destroys sys.executable when launching remote processes - a launched
     python process depends on sys.executable to effectively evaluate its
     module search paths. Without it, regardless of which python interpreter you launch
     you will get the to built in module search paths.
 
-    We use the ip{cluster, engine, controller} scripts as our executable to circumvent 
-    this - the mechanism of shebanged scripts means that the python binary will be 
-    launched with argv[0] set to the *location of the ip{cluster, engine, controller} 
+    We use the ip{cluster, engine, controller} scripts as our executable to circumvent
+    this - the mechanism of shebanged scripts means that the python binary will be
+    launched with argv[0] set to the *location of the ip{cluster, engine, controller}
     scripts on the remote node*. This means you need to take care that:
 
     a. Your remote nodes have their paths configured correctly, with the ipengine and ipcontroller
@@ -1361,7 +1464,7 @@ class HTCondorLauncher(BatchSystemLauncher):
             self.batch_template = '\n'.join([self.batch_template,
                 self.job_array_template])
 
-    def _insert_queue_in_script(self):
+    def _insert_options_in_script(self):
         """AFAIK, HTCondor doesn't have a concept of multiple queues that can be
         specified in the script.
         """
@@ -1455,6 +1558,11 @@ pbs_launchers = [
     PBSControllerLauncher,
     PBSEngineSetLauncher,
 ]
+slurm_launchers = [
+    SlurmLauncher,
+    SlurmControllerLauncher,
+    SlurmEngineSetLauncher,
+]
 sge_launchers = [
     SGELauncher,
     SGEControllerLauncher,
@@ -1470,5 +1578,6 @@ htcondor_launchers = [
     HTCondorControllerLauncher,
     HTCondorEngineSetLauncher,
 ]
-all_launchers = local_launchers + mpi_launchers + ssh_launchers + winhpc_launchers\
-                + pbs_launchers + sge_launchers + lsf_launchers + htcondor_launchers
+all_launchers = local_launchers + mpi_launchers + ssh_launchers + winhpc_launchers +\
+                pbs_launchers + slurm_launchers + sge_launchers + lsf_launchers +\
+                htcondor_launchers
