@@ -633,7 +633,8 @@ class Client(HasTraits):
             self._update_engines(dict(content['engines']))
         else:
             self._connected = False
-            raise Exception("Failed to connect!")
+            tb = '\n'.join(content.get('traceback', []))
+            raise Exception("Failed to connect! %s" % tb)
         
         self._start_io_thread()
     
@@ -1261,6 +1262,76 @@ class Client(HasTraits):
         if error:
             raise error
 
+    def become_distributed(self, targets='all', port=0, nanny=False, scheduler_args=None, **worker_args):
+        """Turn the IPython cluster into a dask.distributed cluster
+
+        Parameters
+        ----------
+
+        targets: target spec (default: all)
+            Which engines to turn into distributed workers.
+        port: int (default: random)
+            Which port
+        nanny: bool (default: False)
+            Whether to start workers as subprocesses instead of in the engine process.
+            Using a nanny allows restarting the worker processes via ``executor.restart``.
+        scheduler_args: dict
+            Keyword arguments (e.g. ip) to pass to the distributed.Scheduler constructor.
+        **worker_args:
+            Any additional keyword arguments (e.g. ncores) are passed to the distributed.Worker constructor.
+
+        Returns
+        -------
+
+        executor: distributed.Executor
+            A distributed.Executor connected to the distributed cluster.
+        """
+        import distributed
+
+        dview = self.direct_view(targets)
+
+        if scheduler_args is None:
+            scheduler_args = {}
+        else:
+            scheduler_args = dict(scheduler_args) # copy
+
+        # Start a Scheduler on the Hub:
+        reply = self._send_recv(self._query_socket, 'become_distributed_request',
+            {'scheduler_args': scheduler_args},
+        )
+        if reply['content']['status'] != 'ok':
+            raise self._unwrap_exception(reply['content'])
+        distributed_info = reply['content']
+
+        # Start a Worker on the selected engines:
+        worker_args['ip'] = distributed_info['ip']
+        worker_args['port'] = distributed_info['port']
+        worker_args['nanny'] = nanny
+
+        # Finally, return an Executor connected to the Scheduler
+        dview.apply_sync(util.become_distributed_worker, **worker_args)
+        executor = distributed.Executor('{ip}:{port}'.format(**distributed_info))
+        return executor
+
+    def stop_distributed(self, targets='all'):
+        """Stop the distributed Scheduler and Workers started by become_distributed.
+
+        Parameters
+        ----------
+
+        targets: target spec (default: all)
+            Which engines to turn into distributed workers.
+        """
+        dview = self.direct_view(targets)
+
+        # Start a Scheduler on the Hub:
+        reply = self._send_recv(self._query_socket, 'stop_distributed_request')
+        if reply['content']['status'] != 'ok':
+            raise self._unwrap_exception(reply['content'])
+
+        # Finally, stop all the Workers on the engines
+        dview.apply_sync(util.stop_distributed_worker)
+
     #--------------------------------------------------------------------------
     # Execution related methods
     #--------------------------------------------------------------------------
@@ -1359,7 +1430,7 @@ class Client(HasTraits):
     # construct a View object
     #--------------------------------------------------------------------------
 
-    def load_balanced_view(self, targets=None):
+    def load_balanced_view(self, targets=None, **kwargs):
         """construct a DirectView object.
 
         If no arguments are specified, create a LoadBalancedView
@@ -1370,12 +1441,14 @@ class Client(HasTraits):
 
         targets: list,slice,int,etc. [default: use all engines]
             The subset of engines across which to load-balance execution
+        kwargs: passed to LoadBalancedView
         """
         if targets == 'all':
             targets = None
         if targets is not None:
             targets = self._build_targets(targets)[1]
-        return LoadBalancedView(client=self, socket=self._task_socket, targets=targets)
+        return LoadBalancedView(client=self, socket=self._task_socket, targets=targets,
+                                **kwargs)
     
     def executor(self, targets=None):
         """Construct a PEP-3148 Executor with a LoadBalancedView
@@ -1394,7 +1467,7 @@ class Client(HasTraits):
         """
         return self.load_balanced_view(targets).executor
 
-    def direct_view(self, targets='all'):
+    def direct_view(self, targets='all', **kwargs):
         """construct a DirectView object.
 
         If no targets are specified, create a DirectView using all engines.
@@ -1411,6 +1484,7 @@ class Client(HasTraits):
 
         targets: list,slice,int,etc. [default: use all engines]
             The engines to use for the View
+        kwargs: passed to DirectView
         """
         single = isinstance(targets, int)
         # allow 'all' to be lazily evaluated at each execution
@@ -1418,7 +1492,8 @@ class Client(HasTraits):
             targets = self._build_targets(targets)[1]
         if single:
             targets = targets[0]
-        return DirectView(client=self, socket=self._mux_socket, targets=targets)
+        return DirectView(client=self, socket=self._mux_socket, targets=targets,
+                          **kwargs)
 
     #--------------------------------------------------------------------------
     # Query methods
