@@ -3,14 +3,12 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import print_function
+from __future__ import absolute_import, print_function
 
 import imp
 import warnings
 from contextlib import contextmanager
 
-import zmq
-
 from .. import serialize
 from traitlets import (
     HasTraits, Any, Bool, List, Dict, Set, Instance, CFloat, Integer
@@ -493,7 +491,6 @@ class DirectView(View):
         
         This calls ipyparallel.serialize.use_dill() here and on each engine.
         """
-        print(serialize)
         serialize.use_dill()
         return self.apply(serialize.use_dill)
 
@@ -505,6 +502,14 @@ class DirectView(View):
         serialize.use_cloudpickle()
         return self.apply(serialize.use_cloudpickle)
 
+    def use_pickle(self):
+        """Restore
+        
+        This reverts changes to serialization caused by `use_dill|.cloudpickle`.
+        """
+        serialize.use_pickle()
+        return self.apply(serialize.use_pickle)
+
 
     @sync_results
     @save_ids
@@ -552,7 +557,7 @@ class DirectView(View):
                                     ident=ident)
             futures.append(future)
         if track:
-            trackers = [f.tracker for f in futures]
+            trackers = [_.tracker for _ in futures]
         else:
             trackers = []
         if isinstance(targets, int):
@@ -1103,6 +1108,28 @@ class LoadBalancedView(View):
         pf = ParallelFunction(self, f, block=block, chunksize=chunksize, ordered=ordered)
         return pf.map(*sequences)
 
+    def register_joblib_backend(self, name='ipyparallel', make_default=False):
+        """Register this View as a joblib parallel backend
+
+        To make this the default backend, set make_default=True.
+
+        Use with::
+
+            p = Parallel(backend='ipyparallel')
+            ...
+
+        See joblib docs for details
+
+        Requires joblib >= 0.10
+
+        .. versionadded:: 5.1
+        """
+        from joblib.parallel import register_parallel_backend
+        from ._joblib import IPythonParallelBackend
+        register_parallel_backend(name,
+            lambda : IPythonParallelBackend(view=self),
+            make_default=make_default)
+
 from concurrent.futures import Executor
 
 class ViewExecutor(Executor):
