@@ -3,17 +3,35 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
+from jupyter_core.paths import jupyter_config_dir
 from traitlets.config.manager import BaseJSONConfigManager
 from notebook.services.config import ConfigManager as FrontendConfigManager
 
 
-def install_server_extension(enable=True):
-    """Register ipyparallel clusters tab as a notebook server extension
+def install_extensions(enable=True):
+    """Register ipyparallel clusters tab as notebook extensions
     
     Toggle with enable=True/False.
     """
+    from distutils.version import LooseVersion as V
+    import notebook
+    
+    if V(notebook.__version__) < V('4.2'):
+        return _install_extension_nb41(enable)
+    
+    from notebook.nbextensions import install_nbextension_python, enable_nbextension, disable_nbextension
+    from notebook.serverextensions import toggle_serverextension_python
+    toggle_serverextension_python('ipyparallel.nbextension')
+    install_nbextension_python('ipyparallel')
+    if enable:
+        enable_nbextension('tree', 'ipyparallel/main')
+    else:
+        disable_nbextension('tree', 'ipyparallel/main')
+
+def _install_extension_nb41(enable=True):
+    """deprecated, pre-4.2 implementation of installing notebook extension"""
     # server-side
-    server = BaseJSONConfigManager()
+    server = BaseJSONConfigManager(config_dir=jupyter_config_dir())
     server_cfg = server.get('jupyter_notebook_config')
     app_cfg = server_cfg.get('NotebookApp', {})
     server_extensions = app_cfg.get('server_extensions', [])
@@ -40,3 +58,4 @@ def install_server_extension(enable=True):
         }
     })
 
+install_server_extension = install_extensions
